package envoy.server.processors;

import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import envoy.data.Contacts;
import envoy.data.LoginCredentials;
import envoy.data.Message.MessageStatus;
import envoy.data.User;
import envoy.data.User.UserStatus;
import envoy.server.ConnectionManager;
import envoy.server.ObjectProcessor;
import envoy.server.data.Message;
import envoy.server.database.PersistenceManager;
import envoy.server.net.ObjectWriteProxy;

/**
 * This {@link ObjectProcessor} handles {@link LoginCredentials}.<br>
 * <br>
 * Project: <strong>envoy-server-standalone</strong><br>
 * File: <strong>LoginCredentialProcessor.java</strong><br>
 * Created: <strong>30.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @author Maximilian K&auml;fer
 * @since Envoy Server Standalone v0.1-alpha
 */
public class LoginCredentialProcessor implements ObjectProcessor<LoginCredentials> {

	private PersistenceManager persistenceManager = PersistenceManager.getPersistenceManager();

	@Override
	public Class<LoginCredentials> getInputClass() { return LoginCredentials.class; }

	@Override
	public void process(LoginCredentials input, long socketId, ObjectWriteProxy writeProxy) throws IOException {
		UserStatusChangeProcessor.setWriteProxy(writeProxy);
		System.out.println(String.format("Received login credentials %s from socket ID %d", input, socketId));

		envoy.server.data.User user = getUser(input);

		// Not logged in successfully
		if (user == null) return;
		ConnectionManager.getInstance().registerUser(user.getId(), socketId);

		// notifies contacts of this users online-going and updates his status in the
		// database
		user.setStatus(UserStatus.ONLINE);
		UserStatusChangeProcessor.updateUserStatus(user);

		// Create contacts
		Contacts contacts = new Contacts(user.getId(),
				user.getContacts().stream().map(envoy.server.data.User::toCommonUser).collect(Collectors.toList()));

		// Complete handshake
		System.out.println("Sending user...");
		writeProxy.write(socketId, user.toCommonUser());
		System.out.println("Sending contacts...");
		writeProxy.write(socketId, contacts);
		System.out.println("Acquiring pending messages for the client...");
		List<Message> pendingMessages = PersistenceManager.getPersistenceManager().getUnreadMessages(user);
		for (Message msg : pendingMessages) {
			System.out.println("Sending message " + msg.toCommonMessage().toString());
			writeProxy.write(socketId, msg.toCommonMessage());
			msg.setReceivedDate(new Date());
			msg.setStatus(MessageStatus.RECEIVED);
			PersistenceManager.getPersistenceManager().updateMessage(msg);
		}
	}

	private envoy.server.data.User getUser(LoginCredentials credentials) {
		envoy.server.data.User user;

		if (credentials.isRegistration()) {
			user = new envoy.server.data.User();
			user.setName(credentials.getName());
			user.setLastSeen(new Date());
			user.setStatus(User.UserStatus.ONLINE);
			user.setPasswordHash(credentials.getPasswordHash());
			user.setContacts(PersistenceManager.getPersistenceManager().getContacts(user));
			persistenceManager.addUser(user);
		} else {
			user = persistenceManager.getUserByName(credentials.getName());
			// TODO: Implement error when user does not exist
			if (!Arrays.equals(credentials.getPasswordHash(), user.getPasswordHash())) // TODO: Wrong Password Response
				return null;
		}
		return user;
	}
}
